#ifndef _KBMAIN_C_
#define _KBMAIN_C_

#include "ifFactory.h"
#include "kbMain.h"
#include "ifMain.h"
#include "kbTrack.h"
#include "kbNote.h"
#include "partFunctions/kbPartExtension.h"

#include <stdlib.h>
#include <string.h>
#include <iostream.h>
#include <sys/ioctl.h>
#include <unistd.h>

#include <stdio.h>
#include <fcntl.h>

#include "globalStuff.c"
#include <math.h>

#include "sound.h"

// childprocess:

#include "childinit.c"

#include "midiFile.c"

#define INITTRACKS 16


KbMain::KbMain( IfFactory * fac, int argc, char ** argv )
  : ifFactory(fac), snap(192), tempo(100), meter0(4), meter1(2), track(0), position(0),
    playWait(false), beepFlag(false), preBeats(2), partShow(0), pp(0), devOpen(false),
    beepMidi(false), clickInst(40), clickVol(120), cycle(false), selX1(0), selX2(4*1536),
    extMan(new KbExtensionManager())
{
  // MIDI init:
  fmOut::setFMPatchesDirectory("/opt/kde/share/apps/kooBase/fm");
  MidiMapper *map=new MidiMapper(NULL); // "/opt/kde/share/apps/kmid/maps/gm.map"); // YamahaPSS790.map");
  if (!map->OK()) { cout << "PANIC" << endl; exit(0); }
  
  midi=new DeviceManager(0); // defaults to the first entry for all 16 MIDI channels
  midi->initManager();       // defines the devices (one for each entry), creates the midiOut objects
  midi->setMidiMap(map);     // sets the MIDI map
  char * line; int n = midi->numberOfSynthDevices() + midi->numberOfMidiPorts(); // my AWE has 2 synths and 1 MIDI
  for (int i=0; i<n && i<MAXDEVNUM; i++) { devices[i] = strdup(midi->name(i)); line = devices[i]; line+=9; line[0]=0; }

  // init for child process:
  pid_t pid;
  static struct sigaction sigcld;
  sigcld.sa_handler = sigcldHandler; // call to sigcldHandler() when child exits, to avoid zombies
  sigemptyset(&sigcld.sa_mask);
  sigcld.sa_flags = 0;
  sigaction(SIGCHLD, &sigcld, NULL);

  BBT(position);

  ifMain = fac->createMain(this,argc,argv);

  // commandline arguments:
  
  if (argc<2) { // if no arguments given, create std layout
    track = new KbTrack(this,0);
    KbTrack * tr = track;
    for (int i=1;i<INITTRACKS;i++) {
      tr->sNext(new KbTrack(this,i));
      tr = tr->gNext();
    }
  } else { // otherwise load song
    char ext[3];
    int l=strlen(argv[1])-1;
    ext[0] = argv[1][l-2];  ext[1] = argv[1][l-1];  ext[2] = argv[1][l]; 
    cout << "ext: " << ext << endl;
    if (strcmp(ext,"mid")==0 || strcmp(ext,"MID")==0 || strcmp(ext,"idi")==0 || strcmp(ext,"IDI")==0)
      getMidi(new ifstream(argv[1]));
    else if (strcmp(ext,".kb") || strcmp(ext,".KB")==0)
      getKooBase(new ifstream(argv[1]));
    else cout << "kooBase can only read midi or kooBase file format" << endl;
    adjustRightBorder();
  }
  
  ifMain->exec();
}

void KbMain::adjustRightBorder() {
  rightPosition = 0;
  for (KbTrack * tr=gTrack(); tr!=0; tr=tr->gNext())
    for (KbPart * pt=tr->gPart(); pt!=0; pt=pt->gNext()) {
      if (pt->gLast() != 0)
	if (pt->gLast()->gPos()>rightPosition) rightPosition = pt->gLast()->gPos();
    }
  rightPosition += int(1536.0*gMeter(0)/gMeter(1));
  sRightSel(rightPosition);
}

IfFactory * KbMain::gFactory() {
  return ifFactory;
}

IfMain * KbMain::gInterface() {
  return ifMain;
}

KbTrack * KbMain::gTrack(int n) {
  if (n==0) return track;
  KbTrack * tr = track;
  KbTrack * rv = 0;
  for (;tr!=0 && n!=0;n--) tr = tr->gNext();
  if (tr!=0) rv = tr;
  return rv;
}

void KbMain::sTrack(KbTrack * t) {
  track = t;
}

int KbMain::gSnap() {
  return snap;
}

void KbMain::sSnap(int s) {
  snap = s;
}

int KbMain::gTempo() {
  return tempo;
}

void KbMain::sTempo(int t) {
  tempo = t;
  if (playing==true) {
    char help[20];
    sprintf(help,"%i",tempo);
    write(fdToChild[1], help, 20); // to child process
  }
}

int KbMain::gMeter(int i) {
  switch (i) {
  case 0: return meter0; break;
  case 1: return (int) pow(2,meter1); break;
  }
  return 0;
}

void KbMain::sMeter(int i, int m) {
  switch (i) {
  case 0: meter0 = m; break;
  case 1: meter1 = int(1.0001*log(m)/log(2)); break;
  }
}

KbTrack * KbMain::addTrack() {
  KbTrack * rv = new KbTrack(this);
  if (track==0) track = rv;
  else {
    KbTrack * tr;
    for (tr=track;tr->gNext()!=0;tr=tr->gNext()) {}
    tr->sNext(rv);
  }
  return rv;
}

char * KbMain::gDevice(int i) {
  return devices[i];
}

int KbMain::gLeftSel() {
  return selX1;
}

int KbMain::gRightSel() {
  return selX2;
}

int KbMain::gLeftBar() {
  return int(selX1*1.0*gMeter(1)/(1536.0*gMeter(0)));
}

int KbMain::gRightBar() {
  return int(selX2*1.0*gMeter(1)/(1536.0*gMeter(0)));
}

void KbMain::sLeftSel(int p) {
  if (p!=-1) selX1 = p;
  else selX1 = position;
}

void KbMain::sRightSel(int p) {
  if (p!=-1) selX2 = p;
  else selX2 = position;
}

int KbMain::gPos() {
  return position;
}

void KbMain::sPos(int p, int b, int t) {
  if (b==-1) { // p is the only parameter !
    position = p;
  } else { // p=bar, b=beat, t=ticks
    timeF = 1.0*gMeter(0)/gMeter(1);
    ticksPerBar = int(timeF*1536);
    position = ticksPerBar*(p-1) + ticksPerBar*(b-1)/gMeter(0) + t;
    position = position-(position%snap);
    ticks = t;
    beats = b;
    bars  = p;
  }
  if (position<0) position = 0;
}

int KbMain::gPosOf(int p, int b, int t) {
  // p=bar, b=beat, t=ticks
  int myPos;
  timeF = 1.0*gMeter(0)/gMeter(1);
  ticksPerBar = int(timeF*1536);
  myPos = ticksPerBar*(p-1) + ticksPerBar*(b-1)/gMeter(0) + t;
}

void KbMain::BBT(int pos, int t0, int t1) {
  // position = pos;
  m0 = (t0==0)?gMeter(0):t0;
  m1 = (t1==0)?gMeter(1):t1;
  timeF = 1.0*m0/m1;
  if (timeF==0) cerr << "PANIC: BBT" << endl;
  ticksPerBar = int(timeF*1536);
  ticksPerBeat = ticksPerBar/m0;
  timeWI = pos%ticksPerBar;
  ticks = pos%ticksPerBeat;
  beats = (timeWI-ticks)/ticksPerBeat+1;
  bars = (pos-timeWI)/ticksPerBar+1;
}

int KbMain::gBar(int pos, int t0, int t1) {
  if (pos!=-1) BBT(pos,t0,t1);
  return bars;
}

int KbMain::gBeat(int pos, int t0, int t1) {
  if (pos!=-1) BBT(pos,t0,t1);
  return beats;
}

int KbMain::gTick(int pos, int t0, int t1) {
  if (pos!=-1) BBT(pos,t0,t1);
  return ticks;
}

void KbMain::togCycle() {
  if (cycle==true) cycle = false; else cycle = true;
}


// ******************************************************
//
// PLAY
// ====
//

void KbMain::play(int tracki/*=-1*/, int off/*=0*/, int delt/*=0*/)
{
  if (devOpen==true) closeDev();
  
  gInterface()->playInit();

  playTicks = 0;
  playTicksPB = 0;
  stop = 0;
  numScr=0;
  for (KbTrack * tr=track;tr!=0;tr=tr->gNext()) { numScr++; }

  left = gPos();
  right = gRightSel();
  for (int k=0; k<256; k++) { sus[k]=0; susCh[k]=0; susTr[k]=0; }
  sus[0]=0;
  susp=0;

  int i = 0;
  for (KbTrack * pTrack = track; pTrack!=0; pTrack = pTrack->gNext()) {
    t[i] = pTrack;
    cPart[i] = t[i]->gPart();
    mute[i] = t[i]->gToggle(0);
    if (cPart[i]!=0) {
      cPart[i]->sCur(cPart[i]->gFirst());
      tOff[i] = 1536*cPart[i]->gTime1()/cPart[i]->gTime2()*(cPart[i]->gFrom());
      if (cPart[i]->gCur()==0) { stp[i]=2; }
      else
	{
	  if (cPart[i]->gFirst()->gNext()==0) { stp[i]=1;} else { stp[i]=0;} 
	  for (;(cPart[i]->gCur()!=0)&&(cPart[i]->gCur()->gPos() + tOff[i] < left);cPart[i]->sCur(cPart[i]->gCur()->gNext())) {}
	}
      if (cPart[i]->gCur()==0) { stp[i]=2; } else { 
	if (cPart[i]->gFirst()->gNext()==0) { stp[i]=1;}
	else { // nur diese i werden gespielt ;-)
	  if (playTicks==0) {
	    playTicks = 1536*cPart[i]->gTime1()/cPart[i]->gTime2();
	    playTicksPB = 1536/cPart[i]->gTime2();
	    // playBeats = cPart[i]->time1;
	  }
	}
      }
    } else stp[i]=2;
    // Auch wenn mute, oder aus score heraus: stp[...]=2; :
    if (mute[i]==true) stp[i]=2;
    i++;
  }
  if (tracki!=-1) {
    for (int ii=0; ii<numScr; ii++) if (ii!=tracki) stp[ii]=2;
    left = off; right = off+delt;
    // stp[tracki] = 0;
    tOff[tracki] = 0;
    /* if (t[tracki]->part->first==0) stp[tracki]=2;
    else if (t[tracki]->part->first->gNext()==0) stp[tracki]=1;
    cout << stp[tracki] << endl;
   */
  }

  pp=left;
  
  // -------------------------------------------------------------------------------
   
  if (pipe(fdToChild) == -1)	{ cout << "PANIC: could not create pipe" << endl; }
  if (pipe(fdToParent) == -1)	{ cout << "PANIC: could not create pipe" << endl; }

  pid = fork(); // fd[1]: write, fd[0]: read
  if (pid == 0) {	// The Child
    ::close(fdToChild[1]); // just read
    ::close(fdToParent[0]); // write to parent
    fcntl(fdToChild[0], F_SETFL, O_NONBLOCK);
    
    playEvent(fdToChild[0]);
    exit(0);
  }
  else if (pid > 0) {  // The Parent
    playing = true;
    ::close(fdToChild[0]); // just write
    ::close(fdToParent[1]); // read from child
    fcntl(fdToParent[0], F_SETFL, O_NONBLOCK); // ??
    fback = fdToParent[0];
  }
  else { cout << "We have trouble" << endl; } // pid=-1; fork-error

}

void KbMain::waitForClap() {
  unsigned char waitbuffer[CLAPBUF];
  int dspfd, l, sum;
  bool t=false;

  if ((dspfd=open(DSP_NAME, O_RDONLY, 0))==-1) { cout << "open failed" << endl; t = true; }
  
  
  while(t==false) {
    l=read(dspfd,waitbuffer,CLAPBUF);
    if (l==-1) t = true;
    sum = -CLAPBUF*127;
    for (int i=0; i<CLAPBUF;i++) sum += (int) waitbuffer[i];
    if (abs(sum) > 5) t = true;
  }
  // DOIT: -> QtMain:   stat->changeItem("... playing song", ID_MAIN_STATUS);
}

void KbMain::myBeep() {
  clickVol1 = clickVol;
  if (beepCount%playTicks != 0) clickVol1 = clickVol1 / 2;
  midi->noteOff(9,clickInst,clickVol1);
  midi->noteOn(9,clickInst,clickVol1);
}

bool KbMain::gPlayWait() {
  return playWait;
}

void KbMain::playEvent(int fout) {

  if (playWait==true) waitForClap();
  // if (beepFlag==true) { pp -= playTicks*preBeats; myBeep(); }

  char buffer[CMDLEN + 1];
  ssize_t n_read;
  int newTempo;
  beepCount=pp;

  count = 0;

  for (int i=0;i<numScr;i++) led[i] = 0;

  // initialize sequencer
  // --------------------
  
  for (int i=0;i<numScr;i++) midi->setDevice(t[i]->gChannel(),t[i]->gOutput());
  
  devOpen=true;
  midi->openDev();
  midi->initDev();
  midi->tmrStart();
  
  for (int i=0;i<numScr;i++) {
    if (t[i]->gPart()!=0) {
      if (t[i]->gProgram()==128) { midi->chnPatchChange(t[i]->gChannel(),cPart[i]->gProgram()); }
      else { midi->chnPatchChange(t[i]->gChannel(),t[i]->gProgram());
      }
    }
  }

  bool myCycle = true;
  int stpBAK[numScr];
  for (int i=0;i<numScr;i++) { stpBAK[i] = stp[i]; }
  midiOff = -int(100.0*left/tempo);
  if (beepFlag==true) { midiOff += playTicks*preBeats; pp -= playTicks*preBeats; myBeep(); }
  int timePos;
  needSync=false;
  oldTimePos=0;

  while(myCycle==true) {

    cout << "hi" << endl;

    while (stop<4) {

      timePos = midiOff + int(100.0*pp/tempo);
      //cout << timePos << ", " << left << endl;
      midi->sync();
      // if (pp/192==int(pp/192)) myWait(timePos);
      // midi->wait(timePos);
      //      syncgap++;
      //if (syncgap==48) {
      //	midi->wait(timePos);
      //	syncgap=0;
      //}
      if (needSync==true) { midi->sync(); needSync=false; }
	
      if ((n_read = read(fout, buffer, CMDLEN)) > 0) { // message from Parent
	buffer[n_read] = 0;
	newTempo = atoi(buffer);
	if (newTempo==0) { stop=5; position = pp; }
	if (newTempo>0) tempo = newTempo;
	if (newTempo==-1) {
	  char * ptr = &buffer[3];
	  int muteTog = atoi(ptr);
	  if (stp[muteTog]==0) stp[muteTog]=2;
	  else if (stp[muteTog]==2) {
	    stp[muteTog]=0;
	    for (;(cPart[muteTog]->gCur()!=0)&&(cPart[muteTog]->gCur()->gPos() + tOff[muteTog] < pp);
		 cPart[muteTog]->sCur(cPart[muteTog]->gCur()->gNext())) {}
	    if (cPart[muteTog]->gCur()==0) stp[muteTog]=2;
	  }
	}
      }
      
      if (stop<2) {
	
	kk=0;
	while (sus[kk]!=0) {
	  if (susPos[kk] + sus[kk]->gLen()<=pp+1) {
	    myWait(timePos);
	    midi->noteOff(susCh[kk], sus[kk]->gFreq(), sus[kk]->gVel());
	    led[susTr[kk]] = 0;
	    for (int k=kk; k<susp+1; k++) { sus[k]=sus[k+1]; susCh[k]=susCh[k+1]; susOut[k]=susOut[k+1];
	    susPos[k]=susPos[k+1]; susTr[k]=susTr[k+1]; }
	    sus[susp--]=0;
	    kk--;
	  }
	  kk++;
	}
	
	for (int ttt=0; ttt<numScr; ttt++) if (stp[ttt]<2) if (cPart[ttt] !=0) if (cPart[ttt]->gCur() !=0) {
	  while ((cPart[ttt]->gCur()->gPos() +tOff[ttt] <= pp) && (stp[ttt]<2)) {
	    totVol =  cPart[ttt]->gCur()->gVel()+t[ttt]->gVolume();
	    if (totVol>127) totVol=127; if (totVol<0) totVol=0;
	    myWait(timePos);
	    midi->noteOn(t[ttt]->gChannel(), cPart[ttt]->gCur()->gFreq(), totVol);
	    led[ttt] = 10;
	    susCh[susp] = t[ttt]->gChannel();
	    susOut[susp]= t[ttt]->gOutput();
	    susPos[susp]= cPart[ttt]->gCur()->gPos()+tOff[ttt];
	    susTr[susp] = ttt;
	    sus[susp++] = cPart[ttt]->gCur();
	    if (cPart[ttt]->gCur()->gNext()==0) {
	      if (cPart[ttt]->gNext()==0) {
		stp[ttt]=2;
	      } else { // next part:
		cPart[ttt] = cPart[ttt]->gNext(); tOff[ttt] = 1536*cPart[ttt]->gTime1()/cPart[ttt]->gTime2()*(cPart[ttt]->gFrom());
		cPart[ttt]->sCur( cPart[ttt]->gFirst() );
		if (cPart[ttt]->gFirst()==0) { stp[ttt] = 2; }
		else { if (cPart[ttt]->gFirst()->gNext()==0) { stp[ttt] = 1; } else { stp[ttt] = 0; } }
		if (stp[ttt]!=2) { // maybe change program:
		  if (t[ttt]->gProgram()==128) { midi->chnPatchChange(t[ttt]->gChannel(),cPart[ttt]->gProgram()); }
		}
	      }
	    } else { cPart[ttt]->sCur( cPart[ttt]->gCur()->gNext() ); }
	  }
	} else cout << "PANIC:play" << endl;
	answerParent();
	if (beepFlag==true && pp-beepCount>=playTicksPB) { beepCount += playTicksPB; myBeep(); }
	stop=2; for (int i=0;i<numScr;i++) if (stp[i]<2) stop=0;
	if (stop==2) { pp1=pp; } //pp=-1; }
	pp++; // pp+=tempo; // urspruenglich pp++;
	if (pp>=right) { stop = 2; }
      } else {
	if (stop==2) { pp=pp1; stop = 3; }
	if (cycle == false) {
	  if (susp>0) {
	    kk=0;
	    while (sus[kk]!=0) {
	      if (susPos[kk] + sus[kk]->gLen()<=pp) {
		myWait(timePos);
		midi->noteOff(susCh[kk], sus[kk]->gFreq(), sus[kk]->gVel());
		led[susTr[kk]] = 0;
		for (int k=kk; k<susp+1; k++) { sus[k]=sus[k+1]; susCh[k]=susCh[k+1]; susOut[k]=susOut[k+1];
		susPos[k]=susPos[k+1]; susTr[k]=susTr[k+1]; }
		sus[susp--]=0;
		kk--;
	      } 
	      kk++;
	    }
	    answerParent();
	    pp++; // pp+=tempo; // urspruenglich pp++;
	  } else { if (stop!=5) stop = 4; }
	}
      }
    }
    if (cycle == true) {
      if (stop>4) {
	myCycle = false;
      } else {
	myCycle = true;
	cout << "pp: " << pp << ", right: " << rightPosition << ", left: " << left << endl;
	// midiOff += pp-left-1;
	// midiOff = timePos-int(100.0*left/tempo);
	midi->tmrStop();
	midi->tmrStart();
	pp = left-1;
	stop = 0;
	for (int i=0;i<numScr;i++) {
	  cPart[i] = t[i]->gPart();
	  cPart[i]->sCur(cPart[i]->gFirst());
	  tOff[i] = 1536*cPart[i]->gTime1()/cPart[i]->gTime2()*(cPart[i]->gFrom());
	  stp[i] = stpBAK[i];
	}
      }
    } else { myCycle = false; }
  }

  answerParent();

  //SEQ_DUMPBUF();
  //SEQ_STOP_TIMER();

  //seq_end();


  // We have to make sure parent receives stop signal:
  
  for (int i=0;i<10;i++) {
    sprintf(help,"-1 %i\n",stop);
    write(fdToParent[1], help, 80); // "stop" to parent process
  }
  // midi->closeDev();
  // delete midi;
  
  // ioctl(seqfd,SNDCTL_SEQ_RESET,0);
  // cout << "I now exit" << endl;

  devOpen=false;
}

bool KbMain::Playing() { return playing; }

void KbMain::myWait(int timePos) {
  if (oldTimePos!=timePos) {
    midi->wait(timePos);
    oldTimePos=timePos;
  }
  needSync=true;
}

void KbMain::answerParent() {
  // this message goes from the playing sequencer (child process) to the GUI (parent process)
  
  count++;
  if (count==max(int(tempo/10),24)) {
  //  if (count==12) {
    count = 0;
    answer = 0;
    shift = 1;
    for (i=0;i<numScr;i++) {
      led[i]--;
      if (led[i]>0) answer += shift;
      shift *= 2;
    }
    sprintf(help,"%i %i\0",answer,pp);
    write(fdToParent[1], help, 80); // to parent process
  }
  
}


void KbMain::stopAll() {
    write(fdToChild[1], "0\0", 2); // to child process
    // DOIT:ioctl(seqfd,SNDCTL_SEQ_RESET,0);
}

void KbMain::hit(int out, int ch, int freq, int vol) {
  //midiOut * m = midi->chntodev(ch);
  //m->openDev(open("/dev/sequencer", O_WRONLY, 0));  

  if (devOpen==false) {
    cout << "opening device" << endl;
    midi->setDevice(ch,out);
    midi->openDev();
    midi->initDev();
    devOpen=true;
  }
  //midi->tmrStart();
  if (freq!=0) {
    midi->noteOn(ch,freq,vol);
    midi->relwait(400);
    midi->noteOff(ch,freq,vol);
    midi->sync();
  }
}

void KbMain::closeDev() {
  cout << "closing device" << endl;
  //midi->tmrStop();
  if (devOpen==true)  midi->closeDev();
  ////m->closeDev();
  devOpen=false;
}

void KbMain::muteTrack(int trNum) {
  if (playing==true) {
    char help[20];
    sprintf(help,"%i %i",-1,trNum);
    write(track->gMain()->fdToChild[1], help, 20); // to child process
  }
}

int KbMain::gFback() {
  return fback;
}

int KbMain::gRightPosition() {
  return rightPosition;
}

void KbMain::sRightPosition(int p) {
  rightPosition = p;
}

// getMidi() and saveMidi() are in file midiFile.c

void KbMain::getKooBase(ifstream * inPtr) {
  //cout << "loading kooBase..." << endl;
  
  ifstream &in = *inPtr;

  char * line = new char[201];
  KbTrack * kbtrack;

  in.getline(line,200,'\n'); // "kooBase"
  if (strcmp(line,"kooBase")!=0) {
    // DOIT: stat->changeItem("Not a kooBase file", ID_MAIN_STATUS);
  } else {
    in.getline(line,200,'\n');
    double version = atof(line);
    if (version < 0.95) {
      // DOIT: stat->changeItem("old version number (<0.95)", ID_MAIN_STATUS);
    }
    else {
      KbPart * part; KbPart * pre_part;
      int stop; int freq; int vel; int len; int pos; int enh;
      in.getline(line,200,'\n'); tempo = atoi(line); // TEMPO
      // DOIT: (also MIDI!): adjustTempo();
      in.getline(line,200,'\n'); sMeter(0,atoi(line));
      in.getline(line,200,'\n'); sMeter(1,atoi(line));
      in.getline(line,200,'\n'); int numScr = atoi(line); // Number of Tracks
      // DOIT: progressbar->setTotalSteps(numScr);
      for (int i=0; i<numScr; i++) {
	// DOIT: progressbar->setProgress(i+1);
	// Track: ------------------------------------------
	// KbTrack * tr = mainCanvas->tr[i];
	kbtrack = addTrack();
	in.getline(line,200,'\n'); kbtrack->sName(line);
	cout << ": " << line << endl; 
	in.getline(line,200,'\n'); kbtrack->sProgram(atoi(line));
	in.getline(line,200,'\n'); kbtrack->sChannel(atoi(line));
	in.getline(line,200,'\n'); kbtrack->sOutput(atoi(line));
	in.getline(line,200,'\n'); kbtrack->sToggle(0,atoi(line));
	in.getline(line,200,'\n'); kbtrack->sToggle(1,atoi(line));
	in.getline(line,200,'\n'); kbtrack->sToggle(2,atoi(line));
	in.getline(line,200,'\n'); kbtrack->sVolume(atoi(line));
	in.getline(line,200,'\n'); kbtrack->sDelay(atoi(line));
	// Parts: ------------------------------------------
	in.getline(line,200,'\n'); int exist = atoi(line);
	part = 0;
	while (exist != -1) { // track not ended yet
	  pre_part = part;
	  part = kbtrack->addPart(0,4);
	  part->sFrom(exist);
	  in.getline(line,200,'\n'); part->sTo(atoi(line));
	  in.getline(line,200,'\n'); part->sClef(atoi(line));
	  in.getline(line,200,'\n'); part->sTime1(atoi(line));
	  in.getline(line,200,'\n'); part->sTime2(atoi(line));
	  in.getline(line,200,'\n'); part->sKey(atoi(line));
	  in.getline(line,200,'\n'); part->sProgram(atoi(line));
	  // Notes: ----------------------------------------
	  stop=0;
	  while (stop==0) {
	    in.getline(line,200,'\n'); freq=atoi(line);
	    if (freq==-2) { stop = 1; } else {
	      in.getline(line,200,'\n'); vel=atoi(line);
	      in.getline(line,200,'\n'); len=atoi(line);
	      in.getline(line,200,'\n'); pos=atoi(line);
	      in.getline(line,200,'\n'); enh=atoi(line);
	      part->appendNote(new KbNote(freq,vel,len,pos,enh));
	    }
	  }
	  in.getline(line,200,'\n'); exist = atoi(line);
	}
      } 
      in.close();
    }
  }
  delete line;
  adjustRightBorder();
  gInterface()->update();
}

void KbMain::saveKooBase(ofstream * outPtr) {
  //cout << "writing kooBase..." << endl;

  ofstream &to = *outPtr;

  KbTrack * tr;
  numScr=0; for (tr=gTrack();tr!=0;tr=tr->gNext()) { numScr++; }
  tr = gTrack();
  KbPart * pt; KbNote * c;

  to << "kooBase" <<endl;
  to << "0.95";
  to << endl;
  to << gTempo() << endl;
  to << gMeter(0) << endl;
  to << gMeter(1) << endl;
  to << numScr << endl;
  // DOIT: progressbar->setTotalSteps(numScr);
  for (tr=gTrack();tr!=0;tr=tr->gNext()) {
    // progressbar->setProgress(i+1);
    // TRACK
    to << tr->gName() << endl;
    // to << tr->instbox->value() << endl;
    to << tr->gProgram() << endl;
    to << tr->gChannel() << endl;
    // to << tr->output->value() << endl;
    to << tr->gOutput() << endl;
    to << tr->gToggle(0) << endl;
    to << tr->gToggle(1) << endl;
    to << tr->gToggle(2) << endl;
    to << tr->gVolume() << endl;
    to << tr->gDelay() << endl;
    // PARTS
    pt = tr->gPart();
    while (pt!=0) {
      to << pt->gFrom() << endl;
      to << pt->gTo() << endl;
      to << pt->gClef() << endl;
      to << pt->gTime1() << endl;
      to << pt->gTime2() << endl;
      to << pt->gKey() << endl;
      // to << pt->trAdr->channel << endl;
      to << pt->gProgram() << endl;
      // to << pt->name << endl;
      
      for (c=pt->gFirst();c!=0;c=c->gNext())
	{
	  to << c->gFreq() << endl;
	  to << c->gVel() << endl;
	  to << c->gLen() << endl;
	  to << c->gPos() << endl;
	  to << c->gEnh() << endl;
	}
      pt = pt->gNext();
      to << -2 << endl; // part end
    }
    to << -1 << endl; // track end
  }
  
  to.close();
}

void KbMain::clear() {
  KbTrack * t;
  KbTrack * t1;
  t = track;
  while (t!=0) {
    if (t->gNext()==0) { delete t; t = 0; }
    else { t1 = t; t = t->gNext(); delete t1; }
  }
  track = 0;
}



bool KbMain::gBeepFlag() {
  return beepFlag;
}

bool KbMain::gBeepMidi() {
  return beepMidi;
}

int KbMain::gClickInst() {
  return clickInst;
}

int KbMain::gClickVol() {
  return clickVol;
}
  
int KbMain::gPreBeats() {
  return preBeats;
}


void KbMain::sBeepFlag(bool b) { beepFlag = b; }

void KbMain::sBeepMidi(bool b) { beepMidi = b; }

void KbMain::sClickInst(int i) { clickInst = i; }

void KbMain::sClickVol(int i) { clickVol = i; }

void KbMain::sPreBeats(int i) { preBeats = i; }

void KbMain::sPlayWait(bool b) { playWait = b; }

KbExtensionManager * KbMain::extManager() { return extMan; }

#endif
