#ifndef _QTDRUM_C_
#define _QTDRUM_C_

#include "qtDrum.h"
#include "kbTrack.h"
#include "kbPart.h"
#include "kbNote.h"

// Drums 35-81

const char * gmDrums[] = {"Acoustic Bass Drum", "Bass Drum 1", "Side Stick", "Acoustic Snare", "Hand Clap", "Electric Snare",
"Low Floor Tom", "Closed Hi-Hat", "High Floor Tom", "Pedal Hi-Hat", "Low Tom", "Open Hi-Hat", "Low-Mid Tom", "Hi-Mid Tom",
"Crash Cymbal 1", "High Tom", "Ride Cymbal 1", "Chinese Cymbal", "Ride Bell", "Tambourine", "Splash Cymbal", "Cowbell",
"Crash Cymbal 2", "Vibraslap", "Ride Cymbal 2", "Hi Bngo", "Low Bongo", "Mute Hi Conga", "Open Hi Conga", "Low Conga",
"Hi Timbale", "Low Timbale", "High Agogo", "Low Agogo", "Cabasa", "Maracas", "Short Whistle", "Long Whistle", "Short Guiro",
"Long Guiro", "Claves", "Hi Wood Block", "Low Wood Block", "Mute Cuica", "Open Cuica", "Mute Triangle", "Open Triangle"};


// ******************************************************************
//
// Constructor
// ===========
//


QtDrum::QtDrum(KbPart * kbpart)
  : QtEditor("Drum",kbpart,"Jan Wrthner")
{
  QtEditor::init();
  xoffset=112;
  yoffset=98;
  setBackgroundMode( PaletteMid );

  QBitmap maske;
  KIconLoader * loader = kapp->getIconLoader();
  img[0] = loader->loadIcon( "drum1.xbm" ); maske = img[0]; img[0].setMask( maske );
  img[1] = loader->loadIcon( "drum2.xbm" ); maske = img[1]; img[1].setMask( maske );
  img[2] = loader->loadIcon( "drum3.xbm" ); maske = img[2]; img[2].setMask( maske );
  img[3] = loader->loadIcon( "drum4.xbm" ); maske = img[3]; img[3].setMask( maske );
  img[4] = loader->loadIcon( "drum5.xbm" ); maske = img[4]; img[4].setMask( maske );
  img[5] = loader->loadIcon( "drum6.xbm" ); maske = img[5]; img[5].setMask( maske );
  img[6] = loader->loadIcon( "drum7.xbm" ); maske = img[6]; img[6].setMask( maske );
  img[7] = loader->loadIcon( "drum8.xbm" ); maske = img[7]; img[7].setMask( maske );

}

void QtDrum::paintEvent( QPaintEvent * pe ) {

  wid = width()-xoffset-16;
  //  QPainter sreal; // smooth animation

  QPixmap pix( width(), height() );
  pix.fill( this, 10,10 );
  QPainter p;
  p.begin( &pix );

  p.drawLine(16,2+yoffset,width()-16,2+yoffset);
  p.drawLine(16,6+47*9+yoffset,width()-16,6+47*9+yoffset);
  p.drawLine(16,2+yoffset,16,6+47*9+yoffset);
  p.drawLine(width()-16,2+yoffset,width()-16,6+47*9+yoffset);
  p.drawLine(xoffset,2+yoffset,xoffset,6+47*9+yoffset);
  p.setFont(QFont("clean", 8));
  for (int i=0;i<=46;i++) p.drawText(20,13+i*9+yoffset,gmDrums[46-i]);

  barlen = wid/barnum;
  for (int i=0;i<barnum;i++) {
    p.drawLine(xoffset+i*barlen,yoffset-10,xoffset+i*barlen,yoffset+47*9);
    help.setNum(i+xmOffset+leftBar+1);
    p.drawText(xoffset+i*barlen+2,yoffset,help);
  }

  p.setPen(colorGroup().foreground());
  beats = part->gTime2();
  for (int i=1;i<barnum*beats;i++) {
    if (i%4!=0) p.drawLine(xoffset+i*barlen/beats,yoffset+1,xoffset+i*barlen/beats,yoffset+47*9);
  }

  p.setPen(colorGroup().foreground());
  xLeft  = xmOffset*1536*part->gTime1()/part->gTime2();
  xRight = xLeft+barnum*1536*part->gTime1()/beats;
  note = part->gFirst();
  int pos;
  while (note!=0 && note->gPos()<xLeft) note = note->gNext();
  while (note!=0 && note->gPos()<xRight) {

    pos = note->gPos();

    if (selNote==note || (selX1!=-1 && selX1 <= pos+leftPos && pos+leftPos <= selX2)) p.setPen(red); // selected Note red !


    yy = yoffset+7+9*(81-note->gFreq());
    xx = xoffset-5+note->gPos()*barlen*beats/(1536*part->gTime1())-barlen*xmOffset;
    p.drawPixmap(xx,yy,img[int(note->gVel()/18)]);

    if (selNote==note || (selX1!=-1 && selX1 <= pos+leftPos && pos+leftPos <= selX2)) p.setPen(black);
    note = note->gNext();
    // cout << xx << ", " << yy << endl;
  }
  if (showY!=0) {
    p.setPen(red);
    p.drawLine(8,showY+yoffset,12,showY+yoffset);
  }


  // SELECTIONS:
  // -----------

  if (grabX1!=0 && abs(grabX1-grabX2)>5) { // draw grab rect
    p.drawRect(grabX1,yoffset-12,grabX2-grabX1,6);
  }


  
  p.end();

  bitBlt( this, 0, 0, &pix );

}


void QtDrum::mousePressEvent ( QMouseEvent * m ) {
  QtEditor::mousePressEvent(m);
  if (showY!=0) {
    x = m->x(); y = m->y()-yoffset;
    yy = int((y-9)/9);
    if (x<xoffset) {
      // soundHit(part->gTrack()->gOutput(),part->gTrack()->gChannel(),81-yy,100);
      main->hit(part->gTrack()->gOutput(),part->gTrack()->gChannel(),81-yy,100);
    }
  }
}



void QtDrum::mouseMoveEvent ( QMouseEvent * m ) {
  QtEditor::mouseMoveEvent(m);
  x = m->x(); y = m->y()-yoffset;
  
  yy = int((y-9)/9);
  showY_old = showY;
  if (yy>=0 && yy<=46) { showY = yy*9+9; } else { showY = 0; }

  QPainter p;
  p.begin(this);
  p.setPen(QColor(colorGroup().mid()));
  p.drawLine(8,showY_old+yoffset,12,showY_old+yoffset);
  p.setPen(red);
  p.drawLine(8,showY+yoffset,12,showY+yoffset);
  p.end();

  // cout << mouseToPos(x) << endl;
  // repaint( FALSE );
}

void QtDrum::mouseReleaseEvent ( QMouseEvent * m ) {
  QtEditor::mouseReleaseEvent(m);
  x = m->x(); y = m->y()-yoffset;
  yy = int((y-9)/9);
  int pos = timeOfMouse(x+2); // +2 to have some space
  int freq = 81-yy;

  if (createNote==TRUE) {
    if (m->button()==LeftButton) {
      if (x>xoffset) {
	soundHit(part->gTrack()->gOutput(),part->gTrack()->gChannel(),freq,velocValue);
	part->addNote(new KbNote(freq,velocValue,ticksOfLen(lenValue),pos-leftPos,0));
	repaint( FALSE );
      }
    }
    if (m->button()==MidButton) {
      KbNote * n;
      for (n=part->gFirst(); n!=0 && (n->gPos()+deltaOff[selSystem]+leftPos<pos || n->gFreq()!=freq); n=n->gNext()) {}
      selNote = n;
      if (selNote!=0) showNoteInfo();
      else showNoteInfoOff();
      repaint( FALSE );
      if (n!=0) soundHit(part->gTrack()->gOutput(),part->gTrack()->gChannel(),n->gFreq(),velocValue); // plays the note
    }
    if (m->button()==RightButton) {
      if (x>xoffset) {
	KbNote * n;
	for (n=part->gFirst(); n!=0 && (n->gPos()+deltaOff[selSystem]+leftPos<pos || n->gFreq()!=freq); n=n->gNext()) {}
	if (n!=0) {
	  part->deleteNote(n);
	  selNote = 0;
	  showNoteInfoOff();
	  repaint( FALSE );
	}
      }
    }
  }
  
}


#endif
