#ifndef _QTEDITOR_C_
#define _QTEDITOR_C_

#include "qtPart.h"
#include "qtEditor.h"
#include "qtSelFrame.h"
#include "kbNote.h"
#include "kbPart.h"
#include "kbTrack.h"
#include "kbMain.h"

#include <qmessagebox.h> 
#include <qaccel.h>

#include <iostream.h>
#include <math.h>

enum { ID_CLOSE, ID_SET, ID_TRANS, ID_Q, ID_LQ, ID_FL, ID_EDIT_CUT, ID_EDIT_COPY, ID_EDIT_PASTE, ID_EDIT_LEFT, ID_EDIT_RIGHT,
       ID_EDIT_UNMARK, ID_EDIT_TRANS, ID_EDIT_Q, ID_EDIT_LQ, ID_EDIT_FL, ID_EDIT_CLEAR, ID_BACK, ID_REWIND, ID_FORWARD,
       ID_PLAY, ID_STOP, ID_PLUS, ID_MINUS, ID_INSPTR };

enum { ID_OPTIONS_AUTHOR };

QtEditor::QtEditor(char * name, KbPart * kbpart, char * a, bool all)
  : KTopLevelWidget(name), part(kbpart), main(kbpart->gTrack()->gMain()), author(a), barnum(int(3*width()/640)), xmOffset(0),
    velocValue(86), snapValue(192), lenValue(4), triValue(false), dotValue(false), time1(kbpart->gTime1()), time2(kbpart->gTime2()),
    xoffset(0), selNote(0), selX1(-1), selX2(0), grabX1(0), grabX2(0), grabY1(0), grabY2(0), mouseDownX(0), mouseDownY(0),
    shftFlag(FALSE), ctrlFlag(FALSE), noteSelection(0), selSystem(0), insPtr(0), allParts(all)
{
  setCaption(name);
  setCursor(QCursor(2));
  setMouseTracking(TRUE);
  setMinimumWidth(640);
  barlen = width()/barnum;
  deltaOff[0] = 0;
  if (all==false) { // one part is edited
    parts = 1;
    leftBar = part->gFrom();
    leftPos = leftBar*int(1536.0*part->gTime1()/part->gTime2());
  } else { // all parts are edited
    parts = 0;
    for (KbTrack * tr=main->gTrack(); tr!=0; tr=tr->gNext()) {
      if (tr->gPart()!=0 && tr->gToggle(0)==FALSE) {
	partList[parts] = tr->gPart();
	// DOIT: partList[parts]->openScore = this;
	parts++;
      }
    }
    if (parts<2)  {
      part = partList[0];
      leftBar = part->gFrom();
      leftPos = leftBar*int(1536.0*part->gTime1()/part->gTime2());
    } else {
      part = partList[0];
      leftBar = partList[0]->gFrom();
      for (int i=1;i<parts;i++) if (partList[i]->gFrom()<leftBar) leftBar = partList[i]->gFrom();
      for (int i=0;i<parts;i++) deltaOff[i] = int((partList[i]->gFrom()-leftBar)*1536.0*partList[i]->gTime1()/partList[i]->gTime2());
      getPartInfo();
      // cout << "leftBar: " << leftBar << ", " << deltaOff[0] << ", " << deltaOff[1] << endl;
      leftPos = leftBar*int(1536.0*part->gTime1()/part->gTime2());
    }


  }
}


QtEditor::~QtEditor() {
  main->closeDev();
}

void QtEditor::resizeEvent( QResizeEvent * re ) {
  KTopLevelWidget::resizeEvent(re);
  barnum = int(3*width()/640);
}

void QtEditor::init() {
  
  KIconLoader * loader = kapp->getIconLoader();
  
  QPopupMenu * file = new QPopupMenu;
  file->insertItem( klocale->translate("Close"), ID_CLOSE);
  file->setAccel(ALT+Key_W,ID_CLOSE);

  QPopupMenu * system = new QPopupMenu;
  system->insertItem( klocale->translate("Settings"), ID_SET);
  system->insertItem( klocale->translate("Transpose"), ID_TRANS);
  system->insertSeparator();
  system->insertItem( klocale->translate("Quantize"), ID_Q);
  system->insertItem( klocale->translate("Length Quantize"), ID_LQ);
  system->insertItem( klocale->translate("Fixed Length"), ID_FL);

  selection = new QPopupMenu;
  selection->insertItem( klocale->translate("Cut"), ID_EDIT_CUT);
  selection->insertItem( klocale->translate("Copy"), ID_EDIT_COPY);
  selection->insertItem( klocale->translate("Paste"), ID_EDIT_PASTE);
  selection->insertSeparator();
  selection->insertItem( klocale->translate("Mark Left"), ID_EDIT_LEFT);
  selection->insertItem( klocale->translate("Mark Right"), ID_EDIT_RIGHT);
  selection->insertItem( klocale->translate("Unmark"), ID_EDIT_UNMARK);
  selection->insertItem( klocale->translate("Transpose"), ID_EDIT_TRANS);
  selection->insertSeparator();
  selection->insertItem( klocale->translate("Quantize"), ID_EDIT_Q);
  selection->insertItem( klocale->translate("Length Quantize"), ID_EDIT_LQ);
  selection->insertItem( klocale->translate("Fixed Length"), ID_EDIT_FL);
  selection->insertSeparator();
  selection->insertItem( klocale->translate("Erase"), ID_EDIT_CLEAR);

  selection->setAccel(ALT+Key_X,ID_EDIT_CUT);
  selection->setAccel(ALT+Key_C,ID_EDIT_COPY);
  selection->setAccel(ALT+Key_V,ID_EDIT_PASTE);
  selection->setAccel(Key_Escape,ID_EDIT_UNMARK);
  selection->setAccel(ALT+Key_Exclam,ID_EDIT_TRANS);
  selection->setAccel(ALT+Key_Q,ID_EDIT_Q);
  selection->setAccel(ALT+Key_L,ID_EDIT_LEFT);
  selection->setAccel(ALT+Key_R,ID_EDIT_RIGHT);
  selection->setAccel(ALT+Key_Backspace,ID_EDIT_CLEAR);

  selection->setItemEnabled(ID_EDIT_CUT,FALSE);
  selection->setItemEnabled(ID_EDIT_COPY,FALSE);
  selection->setItemEnabled(ID_EDIT_PASTE,FALSE);
  selection->setItemEnabled(ID_EDIT_UNMARK,FALSE);
  selection->setItemEnabled(ID_EDIT_TRANS,FALSE);
  selection->setItemEnabled(ID_EDIT_Q,FALSE);
  selection->setItemEnabled(ID_EDIT_LQ,FALSE);
  selection->setItemEnabled(ID_EDIT_FL,FALSE);

  /*  QPopupMenu * tools = new QPopupMenu;
      tools->insertItem( loader->loadIcon( "n8.xpm" ), ID_TOOLS_NOTE);
      tools->insertItem( loader->loadIcon( "n8.xpm" ), ID_TOOLS_NOTE);
      tools->insertItem( loader->loadIcon( "eraser.xpm" ), ID_TOOLS_NOTE);
  */

  snap = new QPopupMenu;
  snap->insertItem( loader->loadIcon( "n1.xpm" ), 1536 );
  snap->insertItem( loader->loadIcon( "n2.xpm" ), 768 );
  snap->insertItem( loader->loadIcon( "n4.xpm" ), 384 );
  snap->insertItem( loader->loadIcon( "n8.xpm" ), 192 );
  snap->insertItem( loader->loadIcon( "n16.xpm" ), 96 );
  snap->insertItem( loader->loadIcon( "n32.xpm" ), 48 );
  snap->insertItem( loader->loadIcon( "n64.xpm" ), 24 );
  snap->insertItem( klocale->translate("off   "), 1 );
  snap->setCheckable(TRUE);
  snap->setItemChecked(192,TRUE);

  veloc = new QPopupMenu;
  veloc->insertItem( klocale->translate("Off "), 0);
  veloc->insertItem( klocale->translate("ppp "), 14);
  veloc->insertItem( klocale->translate("pp "), 28);
  veloc->insertItem( klocale->translate("p "), 42);
  veloc->insertItem( klocale->translate("mp "), 56);
  veloc->insertItem( klocale->translate("mf "), 70);
  veloc->insertItem( klocale->translate("f "), 86);
  veloc->insertItem( klocale->translate("ff "), 102);
  veloc->insertItem( klocale->translate("fff "), 127);
  veloc->setCheckable(true);
  veloc->setItemChecked(86,TRUE);

  options = new QPopupMenu;
  addOptions();
  options->insertSeparator();
  options->insertItem( klocale->translate("author"), ID_OPTIONS_AUTHOR );

  connect(file,SIGNAL(activated(int)),SLOT(fileMenu(int)));
  connect(system,SIGNAL(activated(int)),SLOT(systemMenu(int)));
  connect(selection,SIGNAL(activated(int)),SLOT(selectionMenu(int)));
  connect(snap,SIGNAL(activated(int)),SLOT(snapMenu(int)));
  connect(veloc,SIGNAL(activated(int)),SLOT(velocMenu(int)));
  connect(options,SIGNAL(activated(int)),SLOT(optionsMenu(int)));

  menu = new KMenuBar( this );
  CHECK_PTR( menu );
  menu->insertItem(klocale->translate("File"),file);
  menu->insertItem(klocale->translate("System"),system);
  menu->insertItem(klocale->translate("Selection"),selection);
  menu->insertItem(klocale->translate("Grid"),snap);
  menu->insertItem(klocale->translate("Dynamics"),veloc);
  menu->insertItem(klocale->translate("Options"),options);
  menu->show();

  setMenu( menu );

  panel = new KToolBar( this );

  panel->insertButton( loader->loadIcon( "start.xpm"), ID_BACK, TRUE, klocale->translate("Back to Start"));
  panel->insertButton( loader->loadIcon( "back.xpm"), ID_REWIND, TRUE, klocale->translate("Rewind"));
  panel->insertButton( loader->loadIcon( "forward.xpm"), ID_FORWARD, TRUE, klocale->translate("Forward"));
  panel->insertSeparator();
  panel->insertButton( loader->loadIcon( "editcut.xpm"), ID_EDIT_CUT, TRUE, klocale->translate("Cut"));
  panel->insertButton( loader->loadIcon( "editcopy.xpm"), ID_EDIT_COPY, TRUE, klocale->translate("Copy"));
  panel->insertButton( loader->loadIcon( "editpaste.xpm"), ID_EDIT_PASTE, TRUE, klocale->translate("Paste"));
  panel->insertButton( loader->loadIcon( "edittrans.xpm"), ID_EDIT_TRANS, TRUE, klocale->translate("Transpose"));
  panel->insertSeparator();
  panel->insertButton( loader->loadIcon( "play.xpm"), ID_PLAY, TRUE, klocale->translate("Play"));
  panel->insertButton( loader->loadIcon( "stop.xpm"), ID_STOP, TRUE, klocale->translate("Stop"));
  panel->insertButton( loader->loadIcon( "hPlus.xpm"), ID_PLUS, TRUE, klocale->translate("Enlarge View"));
  panel->insertButton( loader->loadIcon( "hMinus.xpm"), ID_MINUS, TRUE, klocale->translate("Shrink View"));
  panel->insertSeparator();
  panel->insertButton( loader->loadIcon( "exit.xpm"), ID_CLOSE, TRUE, klocale->translate("close"));
  panel->insertSeparator();

  panel->insertWidget(-2,38,new QLabel("range:",panel));
  selFrame = new QtSelFrame(panel,"-");
  selFrame->show();
  panel->insertWidget(-2,72,selFrame);

  connect(panel,SIGNAL(clicked(int)), SLOT(panelMenu(int)));
  addToolBar( panel );
  panel->show();

  panel->setItemEnabled(ID_EDIT_CUT,FALSE);
  panel->setItemEnabled(ID_EDIT_COPY,FALSE);
  panel->setItemEnabled(ID_EDIT_PASTE,FALSE);
  panel->setItemEnabled(ID_EDIT_TRANS,FALSE);


  panel->insertWidget(-2,32,new QLabel("insert:",panel));
  insLine = new QLineEdit( panel,"-" );
  insLine->setText("1 .1 .0");
  insLine->show();
  insLine->setEnabled(TRUE);
  panel->insertWidget(ID_INSPTR,78,insLine);
  connect(insLine,SIGNAL(returnPressed()),SLOT(setIns()));


  notebar = new Notebar(this);
  //  notebar->setFrameStyle( QFrame::Panel | QFrame::Raised );
  // notebar->setGeometry(0,56,640,22); // next one: 0,78,640,22
  addToolBar(notebar);
  notebar->show();

  showNoteInfoOff();

  // seq_init();

  // take it out again: (workarounf for slow OSS!)
  main->hit(part->gTrack()->gOutput(),part->gTrack()->gChannel(),0,0);
}

void QtEditor::getPartInfo() {
  if (parts>1) part = partList[selSystem];
  time1 = part->gTime1();
  time2 = part->gTime2();
}


int QtEditor::gLenValue() {
  return lenValue;
}

void QtEditor::sLenValue(int l) {
  lenValue = l;
}

bool QtEditor::gDotValue() {
  return dotValue;
}

void QtEditor::sDotValue(bool d) {
  dotValue = d;
}

bool QtEditor::gTriValue() {
  return triValue;
}

void QtEditor::sTriValue(bool t) {
  triValue = t;
}


void QtEditor::addOptions() { }

void QtEditor::fileMenu(int n) {
  switch(n) {
  case ID_CLOSE:
    // seq_end();
    ((QtPart *) part->gInterface())->closeEditor();
    delete this;
  }
}

void QtEditor::systemMenu(int n) {
  switch(n) {
  case ID_SET:
    settings();
    break;
  case ID_TRANS:
    transposeSystem();
    break;
  case ID_Q:
    part->quantize(snapValue);
    break;
  case ID_LQ:
    part->quantizeLength(snapValue);
    break;
  case ID_FL:
    part->fixedLength(snapValue);
    break;
  }
  repaint(FALSE);
}

void QtEditor::selectionMenu(int n) {
  switch(n) {
  case ID_EDIT_CUT:
    cutSelection();
    break;
  case ID_EDIT_COPY:
    copySelection();
    break;
  case ID_EDIT_PASTE:
    pasteSelection();
    break;
  case ID_EDIT_LEFT:
    enableSelItems(TRUE);
    selX1 = insPtr;
    main->BBT(selX1,time1,time2);
    selFrame->setTop(main->gTick(),main->gBeat(),main->gBar());
    if (selX2<=selX1) { selX2 = selX1; selFrame->setBottom(main->gTick(),main->gBeat(),main->gBar()); }
    break;
  case ID_EDIT_RIGHT:
    enableSelItems(TRUE);
    selX2 = insPtr;
    if (selX1<0 || selX1>selX2) { selX1 = 0; selFrame->setTop(0,1,1); }
    main->BBT(selX2,time1,time2);
    selFrame->setBottom(main->gTick(),main->gBeat(),main->gBar());
    break;
  case ID_EDIT_UNMARK:
    unmarkSelection();
    break;
  case ID_EDIT_TRANS:
    transposeSelection();
    break;
  case ID_EDIT_Q:
    part->quantize(snapValue,selX1,selX2);
    break;
  case ID_EDIT_LQ:
    part->quantizeLength(snapValue,selX1,selX2);
    break;
  case ID_EDIT_FL:
    part->fixedLength(snapValue,selX1,selX2);
    break;
  case ID_EDIT_CLEAR:
    if (selNote==0) {
      clearSelection();
    } else {
      part->deleteNote(selNote);
      selNote=0;
    }
    break;
  }
  repaint(FALSE);
}

void QtEditor::optionsMenu(int n) {
  switch(n) {
  case ID_OPTIONS_AUTHOR:
    QMessageBox::information( this, "kooBase Editor", "This editor is written by", author );
  }
}

void QtEditor::snapMenu(int n) {
  snap->setItemChecked(snapValue,FALSE);
  snapValue = n;
  snap->setItemChecked(n,TRUE);
}

void QtEditor::velocMenu(int n) {
  veloc->setItemChecked(velocValue,FALSE);
  velocValue = n;
  veloc->setItemChecked(n,TRUE);
}

void QtEditor::panelMenu(int n) {
  switch(n) {
  case ID_BACK:
    xmOffset = 0;
    break;
  case ID_REWIND:
    if (xmOffset>0) xmOffset--;
    break;
  case ID_FORWARD:
    xmOffset++;
    break;

  case ID_EDIT_CUT:
    cutSelection();
    break;
  case ID_EDIT_COPY:
    copySelection();
    break;
  case ID_EDIT_PASTE:
    pasteSelection();
    break;
  case ID_EDIT_TRANS:
    transposeSelection();
    break;

  case ID_PLAY:
    // DOIT: part = scanvas->pcanvas->selPart[scanvas->selSystem];
    if (part!=0) {
      int mm = 1536*part->gTime1()/part->gTime2();
      main->play(part->gTrack()->trackNum(),xmOffset*mm,barnum*mm);
    }    
    break;
  case ID_STOP:
    main->stopAll();
    break;
  case ID_PLUS:
    if (barnum>1) barnum--;
    break;
  case ID_MINUS:
    barnum++;
    break;
  case ID_CLOSE:
    // seq_end();
    ((QtPart *) part->gInterface())->closeEditor();
    delete this;
    break;
  }
  if (n!=ID_CLOSE) repaint(FALSE);
}


void QtEditor::settings() { }


KbNote * QtEditor::selectedNote() {
  return selNote;
}

void QtEditor::selectNote(KbNote * n) {
  selNote = n;
}

KbPart * QtEditor::gPart() {
  return part;
}


int QtEditor::timeOfMouse(int mPos) { // to use, define xoffset, xmOffset, barlen, time1, time2, snapValue, triValue
  if (mPos<xoffset) return -1;
  int oldSnap = snapValue;
  if (triValue) snapValue = int(snapValue *(2.000001/3.0));
  double deltax = barlen*time2*snapValue/(1536.0*time1); // snap was ticksOfLen(curlen)
  int pos  = snapValue * int((mPos-xoffset)/deltax);
  pos += int((xmOffset+leftBar)*1536.0*part->gTime1()/part->gTime2());
  snapValue = oldSnap;
  return pos;
}

int QtEditor::ticksOfLen( int cl ) {

  int tt = 3*int(pow(2,10-cl));
  if (dotValue==TRUE) tt = int( tt * 1.5 );
  if (triValue==TRUE) tt = int( tt * (2.000001/3.0) );
  return tt;
}

void QtEditor::showNoteInfo() {
  // DOIT: int pos = selNote->gPos()+deltaOff[selSystem];
  int pos = selNote->gPos()+deltaOff[selSystem]+leftPos;

  main->BBT(pos);
  
  notebar->setStart(main->gBar(),main->gBeat(),main->gTick()); // start
  notebar->setLength(selNote->gLen());                         // length
  notebar->setPitch(selNote->gFreq());                         // pitch
  notebar->setVelocity(selNote->gVel());                       // velocity
  notebar->setEnh(selNote->gEnh()+2);                          // Enharmonic shift

  selection->setItemEnabled(ID_EDIT_UNMARK,TRUE);
}

void QtEditor::showNoteInfoOff() {
  notebar->clear();
  if (selX1==-1) selection->setItemEnabled(ID_EDIT_UNMARK,FALSE);
}


void QtEditor::unmarkSelection() {
  enableSelItems(FALSE);
  selNote = 0;
  selX1 = -1;
  selX2 = 0;
  showNoteInfoOff();
}

void QtEditor::transposeSelection() {
  cout << "not implemented yet" << endl;
}

void QtEditor::transposeSystem() {
  cout << "not implemented yet" << endl;
}

void QtEditor::deleteBuffer() {
  KbNote * n; KbNote * n1;
  if (noteSelection != 0) {
    n = noteSelection->gNext();
    n1 = noteSelection;
    while (n!=0) { delete n1; n1 = n; n = n->gNext(); }
  }
  noteSelection = 0;
}

void QtEditor::clearSelection() {
  if (selX1!=-1) {
    getPartInfo();
    
    KbNote * n; KbNote * n1; KbNote * n2; KbNote * nDel; KbNote * nPre;
    n = part->gFirst(); nPre = 0; // nPre is the note right before the selection (=0 if selection starts left most)
    while (n!=0 && n->gPos()+deltaOff[selSystem]+leftPos<selX1) { nPre = n; n = n->gNext(); }
    int offx = n->gPos();
    
    while (n!=0 && n->gPos()+deltaOff[selSystem]+leftPos<=selX2) {
      nDel = n;
      n = n->gNext();
      delete nDel;
    }
    if (nPre==0) part->sFirst(n);
    else nPre->sNext(n);
    if (n==0) part->sLast(n);
  }
}

void QtEditor::cutSelection() {
  if (selX1!=-1) {
    getPartInfo();
    deleteBuffer();

    KbNote * n; KbNote * n1; KbNote * n2; KbNote * nDel; KbNote * nPre;
    n = part->gFirst(); nPre = 0; // nPre is the note right before the selection (=0 if selection starts left most)
    while (n!=0 && n->gPos()+deltaOff[selSystem]+leftPos<selX1) { nPre = n; n = n->gNext(); }
    int offx = n->gPos();
    
    while (n!=0 && n->gPos()+deltaOff[selSystem]+leftPos<=selX2) {
      n2 = n1;
      n1 = new KbNote(n->gFreq(), n->gVel(), n->gLen(), n->gPos()+deltaOff[selSystem]-offx, n->gEnh());
      if (noteSelection==0) noteSelection = n1;
      else n2->sNext(n1);
      nDel = n;
      n = n->gNext();
      delete nDel;
    }
    if (nPre==0) part->sFirst(n);
    else nPre->sNext(n);
    if (n==0) part->sLast(n);
  }
  if (noteSelection!=0) {
    panel->setItemEnabled(ID_EDIT_PASTE,TRUE);
    selection->setItemEnabled(ID_EDIT_PASTE,TRUE);
  } else {
    panel->setItemEnabled(ID_EDIT_PASTE,FALSE);
    selection->setItemEnabled(ID_EDIT_PASTE,FALSE);
  }
}

void QtEditor::copySelection() {
  //
  // nicht noteSelection, sondern mainWindow->noteSelection, oder so...
  //
  if (selX1!=-1) {
    getPartInfo();
    deleteBuffer();

    KbNote * n; KbNote * n1; KbNote * n2;
    n = part->gFirst();
    while (n!=0 && n->gPos()+deltaOff[selSystem]+leftPos<selX1) n = n->gNext();
    int offx = n->gPos();
    while (n!=0 && n->gPos()+deltaOff[selSystem]+leftPos<=selX2) {
      n2 = n1;
      n1 = new KbNote(n->gFreq(), n->gVel(), n->gLen(), n->gPos()+deltaOff[selSystem]-offx, n->gEnh());
      if (noteSelection==0) noteSelection = n1;
      else n2->sNext(n1);
      n = n->gNext();
    }
  }
  if (noteSelection!=0) {
    panel->setItemEnabled(ID_EDIT_PASTE,TRUE);
    selection->setItemEnabled(ID_EDIT_PASTE,TRUE);
  } else {
    panel->setItemEnabled(ID_EDIT_PASTE,FALSE);
    selection->setItemEnabled(ID_EDIT_PASTE,FALSE);
  }
}

void QtEditor::pasteSelection() {
  if (noteSelection!=0) {
    getPartInfo();
    KbNote * n = noteSelection;
    while (n!=0) {
      part->addNote(new KbNote(n->gFreq(), n->gVel(), n->gLen(), n->gPos()+deltaOff[selSystem]+insPtr-leftPos, n->gEnh()));
      n = n->gNext();  
    }
    repaint( FALSE );
    // selX1 = -1;
  }
}


void QtEditor::enableSelItems(bool b) {
  panel->setItemEnabled(ID_EDIT_CUT,b);
  panel->setItemEnabled(ID_EDIT_COPY,b);
  panel->setItemEnabled(ID_EDIT_TRANS,b);
  selection->setItemEnabled(ID_EDIT_CUT,b);
  selection->setItemEnabled(ID_EDIT_COPY,b);
  selection->setItemEnabled(ID_EDIT_TRANS,b);
  if (b==TRUE || selNote==0) selection->setItemEnabled(ID_EDIT_UNMARK,b);
  selection->setItemEnabled(ID_EDIT_Q,b);
  selection->setItemEnabled(ID_EDIT_LQ,b);
  selection->setItemEnabled(ID_EDIT_FL,b);
}


// *****************************************************************************
//
// PAINT EVENT
// ===========
//

void QtEditor::paintEvent( QPaintEvent * pe ) {

  
}

// *****************************************************************************
//
// SLOTS
// =====
//


void QtEditor::mousePressEvent ( QMouseEvent * mouse ) {
  mouseDownX = mouse->x();
  mouseDownY = mouse->y();
}


void QtEditor::mouseMoveEvent  ( QMouseEvent * mouse ) {
  mouseX = mouse->x();
  mouseY = mouse->y();

  int tp = timeOfMouse(mouseX);
  if (tp!=-1) {
    main->BBT(tp);
    // setCursor(QCursor(2));
    if (mouseDownX != 0) { // draw Rect
      grabX1 = mouseDownX;
      grabX2 = mouseX;
      repaint( FALSE );
    }
  } else {
    // setCursor(QCursor(0));
  }
}


void QtEditor::mouseReleaseEvent ( QMouseEvent * mouse ) {
  int x = mouse->x();

  createNote = false;

  if (abs(x-mouseDownX)<3) { // Note erzeugen
    createNote=true;
    selX1 = -1;
    if (selNote==0) selection->setItemEnabled(ID_EDIT_UNMARK,FALSE);
  } else { // Selection markieren !!
    selX1 = timeOfMouse(mouseDownX); //+leftBar*int(1536.0*time1/time2);
    selX2 = timeOfMouse(x); //+leftBar*int(1536.0*time1/time2);
    // cout << "selection: " << selX1 << " - " << selX2 << ":" << leftBar << endl;
    if (selX1>=selX2) {
      enableSelItems(FALSE);
      selX1 = -1;
    } else {
      enableSelItems(TRUE);
      main->BBT(selX1,time1,time2);
      selFrame->setTop(main->gTick(),main->gBeat(),main->gBar());
      main->BBT(selX2,time1,time2);
      selFrame->setBottom(main->gTick(),main->gBeat(),main->gBar());
    }
    grabX1 = 0;
    repaint(FALSE);
  }

  mouseDownX = 0;
  mouseDownY = 0;
  grabX1 = 0;
  grabX2 = 0;

  //  repaint( FALSE );
}

void QtEditor::setIns() {
  // this is called when a new position is entered in the text field
  QString qs = insLine->text();
  qs = qs.simplifyWhiteSpace();
  int c1 = qs.find('.');
  if (c1 != -1) {
    int c2 = qs.find('.',c1+1);
    if (c2 != -1) insPtr = main->gPosOf(atoi(qs.left(c1)), atoi(qs.mid(c1+1,c2-c1-1)), atoi(qs.right(qs.length()-c2-1)));
  }
  if (insPtr<0) insPtr = 0;
  main->BBT(insPtr);
  qs.sprintf( " %d. %d. %d",main->gBar(),main->gBeat(),main->gTick());
  insLine->setText(qs);
  insLine->clearFocus();
  
}

void QtEditor::setIns(int pos) {
  insPtr = pos;
  // DOIT: KbPart * p = pcanvas->selPart[scanvas->selSystem];

  QString qs;

  if (insPtr<0) insPtr = 0;
  main->BBT(insPtr,part->gTime1(),part->gTime2());
  qs.sprintf( " %d. %d. %d",main->gBar(),main->gBeat(),main->gTick());

  insLine->setText(qs);
  insLine->clearFocus();
}

int QtEditor::gLeftDelta() {
  return leftPos+deltaOff[selSystem];
}


void QtEditor::showFreq() {

}

void QtEditor::keyPressEvent ( QKeyEvent * key ) {
  switch( key->key() ) {
  case Key_Space:
    if (key->state()==AltButton) {
      QApplication::setOverrideCursor( arrowCursor );
      setIns(timeOfMouse(mouseX));
    }
    break;
  case Key_Shift:
    shftFlag = TRUE;
    ctrlFlag = FALSE;
    showFreq();
    break;
  case Key_Control:
    ctrlFlag = TRUE;
    shftFlag = FALSE;
    showFreq();
    break;
  case Key_Left:
    getPartInfo();
    if (selNote==0) {
      part->sCur(part->gLast()); selNote = part->gCur(); showNoteInfo();
    } else {
      part->sCur(selNote);
      if (part->left()==true) { selNote = 0; showNoteInfoOff(); }
      else { selNote = part->gCur(); showNoteInfo(); }
    }
    if (selNote!=0) {
      main->hit(part->gTrack()->gOutput(),part->gTrack()->gChannel(),selNote->gFreq(),selNote->gVel());
    }
    break;
  case Key_Right:
    getPartInfo();
    if (selNote==0) {
      part->sCur(part->gFirst()); selNote = part->gCur(); showNoteInfo();
    }
    else {
      part->sCur(selNote);
      if (part->right()==true) { selNote = 0; showNoteInfoOff(); }
      else { selNote = part->gCur(); showNoteInfo(); }
    }
    if (selNote!=0) {
      main->hit(part->gTrack()->gOutput(),part->gTrack()->gChannel(),selNote->gFreq(),selNote->gVel());
    }
  break;
    
  case Key_PageUp:
    if (selSystem>0) selSystem--;
    getPartInfo();
    selNote=0;
    break;
  case Key_PageDown:
    if (selSystem<parts-1) selSystem++;
    getPartInfo();
    selNote = 0;
    break;
  case Key_Up:
    getPartInfo();
    if (selNote!=0) {
      part->sCur(selNote);
      selNote->sFreq(selNote->gFreq()+1);
      showNoteInfo();
      main->hit(part->gTrack()->gOutput(),part->gTrack()->gChannel(),selNote->gFreq(),selNote->gVel());
    } else {
      if (selX1!=-1) {
	KbNote * n = part->gFirst();
	while (n!=0 && n->gPos()+deltaOff[selSystem]<selX1) n = n->gNext();
	while (n!=0 && n->gPos()+deltaOff[selSystem]<=selX2) { n->sFreq(n->gFreq()+1); n = n->gNext(); }
      }
    }
    break;
  case Key_Down:
    getPartInfo();
    if (selNote!=0) {
      part->sCur(selNote);
      selNote->sFreq(selNote->gFreq()-1);
      showNoteInfo();
      main->hit(part->gTrack()->gOutput(),part->gTrack()->gChannel(),selNote->gFreq(),selNote->gVel());
    } else {
      if (selX1!=-1) {
	KbNote * n = part->gFirst();
	while (n!=0 && n->gPos()+deltaOff[selSystem]<selX1) n = n->gNext();
	while (n!=0 && n->gPos()+deltaOff[selSystem]<=selX2) { n->sFreq(n->gFreq()-1); n = n->gNext(); }
      }
    }
    break;
  default:
    key->ignore();
  }
  repaint( FALSE );
}


void QtEditor::keyReleaseEvent ( QKeyEvent * key ) {
  switch( key->key() ) {
  case Key_Space:
    if (key->state()==AltButton) {
      QApplication::restoreOverrideCursor();
    }
    break;
  case Key_Shift:
    shftFlag = FALSE;
    showFreq();
    break;
  case Key_Control:
    ctrlFlag = FALSE;
    showFreq();
      break;
  default:
    key->ignore();
  }
}



#endif
