#ifndef _QTPART_C_
#define _QTPART_C_

#include "qtPart.h"
#include "qtPartCanvas.h"
#include "qtTrack.h"
#include "qtScore.h"
#include "qtDrum.h"
#include "qtPiano.h"
#include "qtEvent.h"
#include "kbPart.h"
#include "kbTrack.h"
#include "kbMain.h"
#include "kbNote.h"
#include "partFunctions/kbPartExtension.h"

#include <iostream.h>

extern const char * gmNames[];
extern const char * gmCat[];

QtPart::QtPart(KbPart * kbpart)
  : qtmain((QtMain*)kbpart->gTrack()->gMain()->gInterface()),
    pcanvas(((QtMain*)kbpart->gTrack()->gMain()->gInterface())->gPartCanvas()),
    QLabel((QLabel*)((QtMain*)kbpart->gTrack()->gMain()->gInterface())->gPartCanvas(),"part"),
    part(kbpart)
{
  int n = kbpart->gTrack()->trackNum()+1;
  int left = kbpart->gFrom();
  int width = kbpart->gTo() - left;
  // cout << "qtPart: " << width << ", " << pcanvas->gBarLength() << endl;
  setGeometry(pcanvas->gOffset()+int((qtmain->gBarOff()+left)*pcanvas->gBarLength()),n*(qtmain->gTrackHeight()+1),int(width*pcanvas->gBarLength()),qtmain->gTrackHeight());
  setFrameStyle( QFrame::Panel | QFrame::Raised );
  // setLineWidth(2);
  instPM = new QPopupMenu(  );
  for (int i=0;i<16;i++) {
    instPMs[i] = new QPopupMenu(  );
    instPMs[i]->setCheckable(TRUE);
    for (int k=0;k<8;k++)
      instPMs[i]->insertItem(gmNames[i*8+k],i*8+k);
    connect(instPMs[i],SIGNAL(activated(int)),SLOT(instMenu(int)));
    instPM->insertItem(gmCat[i], instPMs[i]);
  }
  // instPM->setCheckable(TRUE);
  // connect(instPM,SIGNAL(activated(int)),SLOT(instMenu(int)));
  // instPM->setActiveItem(0);

  userDef = new QPopupMenu();
  KbExtensionManager * em = qtmain->gMain()->extManager();
  for(em->init();em->getExtension()!=0;em->incr()) {
    userDef->insertItem(klocale->translate(em->getExtension()->name()));
  }

  connect(userDef,SIGNAL(activated(int)),SLOT(userDefMenu(int)));
  //userDef->connectItem( userDef->insertItem(klocale->translate("shift one octave up")), this, SLOT(shiftOctaveUp()) );

  // #include "partFunctions/insert.C"

  rbmenu = new QPopupMenu;
  rbmenu->setMouseTracking( TRUE );
  rbmenu->connectItem( rbmenu->insertItem(klocale->translate("Edit Score")), this, SLOT(scoreEdit()) );
  rbmenu->connectItem( rbmenu->insertItem(klocale->translate("Edit Drummap")), this, SLOT(drumEdit()) );
  rbmenu->connectItem( rbmenu->insertItem(klocale->translate("Edit Piano Roll")), this, SLOT(pianoRollEdit()) );
  rbmenu->connectItem( rbmenu->insertItem(klocale->translate("Edit Events")), this, SLOT(eventEdit()) );
  // rbmenu->connectItem( rbmenu->insertItem(klocale->translate("Adjust Part Size")), this, SLOT(adjustPartSize()) );
  rbmenu->insertSeparator();
  rbmenu->connectItem( rbmenu->insertItem(klocale->translate("Copy Part")), this, SLOT(partCopy()) );
  rbmenu->connectItem( rbmenu->insertItem(klocale->translate("Glue Parts")), this, SLOT(partGlue()) );
  rbmenu->connectItem( rbmenu->insertItem(klocale->translate("Split Part")), this, SLOT(partSplit()) );
  rbmenu->insertSeparator();
  rbmenu->insertItem(klocale->translate("Instrument"), instPM );
  rbmenu->insertSeparator();
  rbmenu->insertItem(klocale->translate("User Defined Functions"), userDef );
  rbmenu->insertSeparator();
  rbmenu->connectItem( rbmenu->insertItem(klocale->translate("Quantize All")), this, SLOT(partq()) );
  rbmenu->connectItem( rbmenu->insertItem(klocale->translate("Quantize Lengths")), this, SLOT(partql()) );
  rbmenu->connectItem( rbmenu->insertItem(klocale->translate("Fixed Length")), this, SLOT(partfl()) );
  rbmenu->insertSeparator();
  rbmenu->connectItem( rbmenu->insertItem(klocale->translate("Erase Part")), this, SLOT(deletePart()) );

  // rbmenu->connectItem( rbmenu->insertItem(klocale->translate("Test Output")), this, SLOT(testPart()) );

  show();
}

void QtPart::testPart() {
  cout << "from: " << gPart()->gFrom() << ", to: " << gPart()->gTo() << endl;
  if (gPart()->gFirst() != 0) {
    cout << "first: " << gPart()->gFirst()->gPos() << ", last: " << gPart()->gLast()->gPos() << endl;
  }
}

void QtPart::userDefMenu(int n) {
  KbPartExtension * pe = qtmain->gMain()->extManager()->getExtension(n);
  if (pe!=0) pe->act(part);
}

QtPart::~QtPart() {

}

void QtPart::erase() {
  delete this;
}

void QtPart::instMenu(int entry) {
  part->sProgram(entry);

  ((QtTrack*) part->gTrack()->gInterface())->sProgram(128);
  //  if (openScore!=0) openScore->scanvas->repaint( FALSE );
}

void QtPart::deletePart() {
  qtmain->msg("Part deleted");
  gPart()->deletePart();
}


void QtPart::paintEvent( QPaintEvent * pe ) {
  QLabel::paintEvent(pe);

  int ymid = height()/2;
  QPainter painter;
  painter.begin(this);
  painter.setPen(black);
  //  painter.drawLine(0,0,0,2*ymid);

  if (qtmain->gPartShow()==0) { // show names
    painter.drawText(4,16,part->gTrack()->gName());
  }
  
  if (qtmain->gPartShow()==1) { // show events
    double scl = width()*part->gTime2()/((part->gTo()-part->gFrom())*part->gTime1());
    painter.setPen(QColor(40,40,120));
    for (KbNote * n = part->gFirst();n!=0;n=n->gNext()) {
      int xx = int(n->gPos()*1.0*scl/1536);
      int yy = int(n->gVel()*ymid*1.0/120);
      painter.drawLine(xx,ymid-yy,xx,ymid+yy);
    }
  }
  painter.end();
}


// *******************************************************************************
//
// Editors
// =======
//


void QtPart::scoreEdit() {
  //ScoreWindow * sc = new ScoreWindow("score", mw, this, pcanvas);
  //sc->show();
  QtScore * sc = new QtScore(part);
  sc->show();
}

void QtPart::drumEdit() {
  //DrumWindow * dc = new DrumWindow("drum", mw, this);
  //dc->show();
  QtDrum * dr = new QtDrum(part);
  dr->show();
}

void QtPart::pianoRollEdit() {
  QtPiano * pn = new QtPiano(part);
  pn->show();
}

void QtPart::eventEdit() {
  QtEvent * ev = new QtEvent(part);
  ev->show();
}





void QtPart::closeEditor() {
  // stuff that needs to be done when an editor closes goes here!
  adjustPartSize();
}


// *******************************************************************************
//
// Mouse Events
// ============
//


void QtPart::mousePressEvent( QMouseEvent * mouse ) {
  if (mouse->button()==LeftButton) {
    local_x = mapToParent(mouse->pos()).x();
    dx = 0;
    open = true;
  }
  if (mouse->button()==RightButton) {
    splitPoint = mapToParent(mouse->pos()).x();
    rbmenu->popup( mapToGlobal(mouse->pos()), 0 );
  }
}

void QtPart::mouseMoveEvent( QMouseEvent * mouse ) {
  QPoint p = mapToParent(mouse->pos());
  double barlen = pcanvas->gBarLength();
  int xx = p.x()-local_x;
  int dx1 = int(xx/barlen); 
  if (dx<dx1) {
    part->sFrom(part->gFrom()+1);
    part->sTo(part->gTo()+1);
    pcanvas->repaint();
    qtmain->msg("... moving part");
  }
  if (dx>dx1 && part->gFrom()>0) {
    part->sFrom(part->gFrom()-1);
    part->sTo(part->gTo()-1);
    pcanvas->repaint();
    qtmain->msg("... moving part");
  }

  dx=dx1;
  open = false;
}

void QtPart::mouseReleaseEvent( QMouseEvent * )
{
  if (open==false) {
    qtmain->msg("Part moved");
    adjustPartSize();
  }
  // trAdr->checkParts();
}

void QtPart::adjustPartSize() {
  // from, to are measure numbers
  int from = part->gFrom();
  int to = part->gTo();
  KbNote * start = part->gFirst();
  KbNote * end = part->gLast();
  int ticksPM;
  if (start!=0 && end!=0) {
    int fromOld = from;
    int ticksPM = int(1536.0*part->gTime1()/part->gTime2());
    from = from + int(start->gPos()/ticksPM);
    int diff = ticksPM*(from-fromOld);
    if (diff!=0) for (KbNote * n=start;n!=0;n=n->gNext()) n->sPos(n->gPos()-diff);
    to = from + 1 + int(end->gPos()/ticksPM);
    part->sFrom(from);
    part->sTo(to);
    pcanvas->repaint();
    if (to*ticksPM > qtmain->gMain()->gRightPosition()) {
      qtmain->gMain()->sRightPosition(to*ticksPM);
      // mw->posSlider->setRange(0,mw->rightPosition);
    }
  }
}

void QtPart::partCopy() {
  // QtPart * p = new QtPart(trNum,to,from+to,pcanvas);
  KbPart * p = part->gTrack()->addPart(part->gTo(),part->gFrom()+part->gTo());
  //p->trAdr = trAdr;
  //p->next = next;
  //next = p;
  for (KbNote * n=part->gFirst();n!=0;n=n->gNext())
    p->appendNote(new KbNote(n->gFreq(),n->gVel(),n->gLen(),n->gPos(),n->gEnh()));
  // trAdr->checkParts();
}

void QtPart::partGlue() {
  gPart()->glue();
  pcanvas->repaint();
}

void QtPart::partSplit() {
  int splitBar = int((splitPoint-pcanvas->gOffset())/pcanvas->gBarLength()+0.5);
  gPart()->split(splitBar);
  pcanvas->repaint();
}


void QtPart::partq() {
  part->quantize(part->gTrack()->gMain()->gSnap());
}

void QtPart::partql() {
  part->quantizeLength(part->gTrack()->gMain()->gSnap());
}

void QtPart::partfl() {
  part->fixedLength(part->gTrack()->gMain()->gSnap());
}



KbPart * QtPart::gPart() { return part; }

void QtPart::sPart(KbPart * p) { part = p; }

KbPart * QtPart::gNext() {
  return part->gNext();
}

void QtPart::sNext(KbPart * p) {
  part->sNext(p);
}




#endif
