#ifndef _QTPARTCANVAS_C_
#define _QTPARTCANVAS_C_

#include "qtPartCanvas.h"
#include "qtPart.h"
#include "qtScore.h"
#include "kbMain.h"
#include "kbTrack.h"

#include <iostream.h>

// ************************************************
//
// QtPartCanvas
// ============
//


QtPartCanvas::QtPartCanvas( QtMain * qtm, QWidget * parent )
  : QFrame( parent ), qtmain(qtm), xoffset(18)
{
  timebar = new QtTimeBar( qtmain, this );
  timebar->setGeometry(0,0,1600,24);
  timebar->setFrameStyle( QFrame::Panel | QFrame::Raised );
  timebar->setLineWidth(2);
  timebar->show();
  
  rbmenu = new QPopupMenu;
  rbmenu->setMouseTracking( TRUE );
  rbmenu->connectItem( rbmenu->insertItem(klocale->translate("Edit Entire Score")), this, SLOT(scoreEdit()) );
  rbmenu->connectItem( rbmenu->insertItem(klocale->translate("Preferences")), this, SLOT(preferences()) );
  rbmenu->insertSeparator();
  rbmenu->connectItem( rbmenu->insertItem(klocale->translate("Erase Entire Song")), this, SLOT(deleteSong()) );
  
  //  rbmenu->connectItem( rbmenu->insertItem(klocale->translate("")), this, SLOT() );
  line = new QWidget(this);
  line->setBackgroundColor(white);
  line->setBackgroundMode( PaletteMid );
  // line->hide();
  // line->setGeometry(0,0,1,1);
  drawLine();
}

void QtPartCanvas::preferences() {
  qtmain->createPrefs();
}


void QtPartCanvas::scoreEdit() {
  KbPart * pt = 0;
  for (KbTrack * tr=qtmain->gMain()->gTrack(); tr!=0;) {
    if (tr->gPart()!=0) { pt=tr->gPart(); tr=0; }
    if (tr!=0) tr=tr->gNext();
  }
  if (pt!=0) {
    QtScore * sc = new QtScore(pt,true);
    sc->show();
    qtmain->msg("Editing Part");
  } else {
    qtmain->msg("No Parts Available");
  }
}

void QtPartCanvas::deleteSong()
{
  qtmain->gMain()->clear();
}

void QtPartCanvas::paintEvent( QPaintEvent * pe )
{
  timebar->repaint();
  KbTrack * tr = qtmain->gMain()->gTrack();
  int n=0; int left; int wid;
  while (tr!=0) {
    n++;
    if (tr->gPart()!=0) {
      for (QtPart * p=(QtPart*) tr->gPart()->gInterface();p!=0;) {
	left = p->gPart()->gFrom();
	wid = p->gPart()->gTo() - left;
	p->setGeometry(gOffset()+int((qtmain->gBarOff()+left)*gBarLength()),
		       n*(qtmain->gTrackHeight()+1),
		       int(wid*gBarLength()),
		       qtmain->gTrackHeight());
	p->repaint();
	if (p->gNext()!=0) p=(QtPart*) p->gNext()->gInterface();
	else p = 0;
      }
    }
    tr = tr->gNext();
  }
}

int QtPartCanvas::gOffset() {
  return xoffset;
}

int QtPartCanvas::gPos() {
  return qtmain->gPos();
}

void QtPartCanvas::mousePressEvent ( QMouseEvent * mouse ) {
  mouseX = mouse->x();
  mouseY = mouse->y();
  if (mouse->button()==RightButton) {
    rbmenu->popup( mapToGlobal(mouse->pos()), 0 );
  }
}


void QtPartCanvas::mouseDoubleClickEvent( QMouseEvent * mouse ) {
  int yy = mouse->y()/(1+qtmain->gTrackHeight())-1;
  KbTrack * tr = qtmain->gMain()->gTrack(yy);
  if (tr==0) tr = qtmain->gMain()->addTrack();
  int leftBar=0;
  int rightBar=4;
  //  KbPart * p = new KbPart(yy,leftBar,rightBar,tr);
  tr->addPart(leftBar,rightBar);
}


void QtPartCanvas::mouseMoveEvent  ( QMouseEvent * mouse ) {
  // int yy = mouse->y()/(1+qtmain->gTrackHeight())-1;
  // cout << yy << endl;
}


void QtPartCanvas::mouseReleaseEvent ( QMouseEvent * mouse ) {

}

void QtPartCanvas::drawLine() {

  line->setGeometry(gOffset()+int(gPos()*gBarLength()*qtmain->gMain()->gMeter(1)/(1536*qtmain->gMain()->gMeter(0))),1,1,1023);
 
}

double QtPartCanvas::gBarLength() {
  return timebar->gBarLength();
}



// ************************************************
//
// QtTimeBar
// =========
//


QtTimeBar::QtTimeBar( QtMain * qtm, QWidget *parent, const char *name )
  : QLabel( parent, name ), qtmain(qtm), pcanvas((QtPartCanvas*)parent)
{
  setBackgroundMode( PaletteMidlight );
  int xoffset = pcanvas->gOffset();
  int numOfBars = qtmain->gBarNum();
  barlen = 1.0*(1600-18-xoffset)/(1.0*numOfBars);
}

QtPartCanvas * QtTimeBar::gPartCanvas() {
  return pcanvas;
}

void QtTimeBar::paintEvent( QPaintEvent * pe ) {
  // int canvasMarginRight = pcanvas->width()-18-pcanvas->canvasMarginLeft;
  //canvasMarginRight = width()-18;
  //canvasMarginLeft = 0;
  int xoffset = pcanvas->gOffset();
  int numOfBars = qtmain->gBarNum();
  int offBar = qtmain->gBarOff();
  
  int leftMargin;
  int leftBar = qtmain->gMain()->gLeftBar();
  int rightMargin;
  int rightBar = qtmain->gMain()->gRightBar();
  int beats = qtmain->gMain()->gMeter(0);
  barlen = 1.0*(width()-18-xoffset)/(1.0*numOfBars);

  QPainter painter;

  painter.begin(this);

  // draw bars and numbers
  int yy;
  char help[12];
  painter.setPen(QPen(QColor("black"),1));
  for (int i=0; i<numOfBars; i++) {
    yy = 18;
    if (i*1.0/beats==int(i*1.0/beats)) { sprintf(help,"%d",offBar+1+i); painter.drawText(canvasMarginLeft+xoffset+int(i*barlen)+2,16,help); yy = 8; }
    painter.drawLine(canvasMarginLeft+int(i*barlen)+xoffset, yy, canvasMarginLeft+int(i*barlen)+xoffset, 22);
  }
  painter.drawLine(canvasMarginRight, 8, canvasMarginRight, 22);

  // draw left and right margin
  
  leftMargin = int(leftBar*barlen+xoffset);
  rightMargin = int(rightBar*barlen+xoffset);
  painter.setPen(QPen(QColor("red")));
  painter.drawLine(leftMargin+canvasMarginLeft, 2, leftMargin+canvasMarginLeft, 22);
  painter.drawLine(leftMargin+canvasMarginLeft-5, 2, leftMargin+canvasMarginLeft, 2);
  painter.drawLine(rightMargin+canvasMarginLeft, 2, rightMargin+canvasMarginLeft, 22);
  painter.drawLine(rightMargin+canvasMarginLeft, 2, rightMargin+canvasMarginLeft+5, 2);
  
  painter.end();
  
}

double QtTimeBar::gBarLength() {
  return barlen;
}



#endif
