#ifndef _QTTRACK_C_
#define _QTTRACK_C_

#include "kbTrack.h"
#include "qtTrack.h"
#include <iostream.h>

extern const char * gmNames[];

MyLabel::MyLabel( QWidget * parent = 0, const char * name = 0 )
  : QLabel(parent,name), qttrack((QtTrack*)parent)
{
  rbmenu = new QPopupMenu;
  rbmenu->setMouseTracking( TRUE );
  rbmenu->connectItem( rbmenu->insertItem(klocale->translate("Change Trackname")), this, SLOT(changeName()) );
  rbmenu->insertSeparator();
  rbmenu->connectItem( rbmenu->insertItem(klocale->translate("Delete Track")), this, SLOT(delTrack()) );
  
}

void MyLabel::mousePressEvent ( QMouseEvent * mouse ) {
  if (mouse->button()==LeftButton) {
  }
  if (mouse->button()==RightButton) {
    rbmenu->popup( mapToGlobal(mouse->pos()), 0 );
  }
}

void MyLabel::changeName() { 
  le = new QLineEdit(this,qttrack->gTrack()->gName());//,WStyle_Customize | WStyle_NoBorder);
  //le->recreate(this,WStyle_Customize | WStyle_NoBorder,mapToGlobal(this->pos()+QPoint(-68,0)));
  le->setText((char*)qttrack->gTrack()->gName());
  le->setFixedHeight(height()-2); le->setFixedWidth(width()-8);
  // le->setGeometry(mapToGlobal(this->pos()).x(),mapToGlobal(this->pos()).y(),220,40);
  le->show();
  le->setActiveWindow();
  le->setFocus();
  connect(le,SIGNAL(returnPressed()),SLOT(setName()));
}

void MyLabel::setName() {
  qttrack->gTrack()->sName((char*) le->text());
  qttrack->update();
  delete le;
}

void MyLabel::delTrack() {
  qttrack->gQtMain()->msg("Track deleted");
  KbTrack * mytr = qttrack->gTrack();
  KbTrack * tr=0;
  for (tr=qttrack->gQtMain()->gMain()->gTrack(); tr!=0&&tr->gNext()!=mytr; tr=tr->gNext()) {}
  if (tr!=0) {
    tr->sNext(mytr->gNext());
    delete mytr;
  } else {
    qttrack->gQtMain()->gMain()->sTrack(mytr->gNext());
    delete mytr;
  }
  qttrack->gQtMain()->adjustHeight();
}


QtTrack::QtTrack(KbTrack * kbtrack)
  : QWidget((QWidget*)((QtMain*)kbtrack->gMain()->gInterface())->gLeft(),"track"), track(kbtrack)
{
  int n = track->trackNum()+1; // the underlying kbTrack of this qtTrack is not registered yet, that's why +1 !
  int hh = height();
  setGeometry(0,n*(1+hh),width(),hh);

  KIconLoader * loader = kapp->getIconLoader();

  QPixmap * pmLock = new QPixmap(loader->loadIcon( "lock.xpm" )); pmLock->setMask(pmLock->createHeuristicMask());
  QPixmap * pmMute = new QPixmap(loader->loadIcon( "mute.xpm" ));
  QPixmap * pmNote = new QPixmap(loader->loadIcon( "note.xpm" ));
  QPixmap * pmWav  = new QPixmap(loader->loadIcon( "wav.xpm" ));

  mute = new TogBut( 0, track, pmMute, 0, this, "mute" );
  mute->setFrameStyle( QFrame::Panel | QFrame::Raised );
  mute->setGeometry(0,0,24,hh+1);

  type = new TogBut( 1, track, pmWav, pmNote, this, "type" );
  type->setFrameStyle( QFrame::Panel | QFrame::Raised );
  type->setGeometry(24,0,24,hh+1);

  lock = new TogBut( 2, track, pmLock, 0, this, "lock" );
  lock->setFrameStyle( QFrame::Panel | QFrame::Raised );
  lock->setGeometry(48,0,24,hh+1);

  trname = new MyLabel(this);
  trname->setText(track->gName());
  trname->setFrameStyle( QFrame::Panel | QFrame::Raised );
  trname->setGeometry(72,0,96,hh+1);

  instbox = new QComboBox(FALSE,this,"instrument");
  instbox->insertStrList( gmNames );
  instbox->setCurrentItem(track->gProgram());
  instbox->setGeometry(168,0,160,hh+1);
  connect(instbox,SIGNAL(highlighted(int)),SLOT(updateInst(int)));

  output = new QComboBox(FALSE,this,"output");
  output->insertStrList( ((QtMain*)kbtrack->gMain()->gInterface())->gDevList() );
  output->setCurrentItem(track->gOutput());
  output->setGeometry(328,0,96,hh+1);
  connect(output,SIGNAL(highlighted(int)),SLOT(updateOut(int)));

  chanbox = new QSpinBox(0,15,1,this);
  chanbox->setFrameStyle( QFrame::Panel | QFrame::Raised );
  chanbox->setGeometry(424,0,44,hh+1);
  chanbox->setValue(track->gChannel());
  chanbox->setBackgroundMode( PaletteMid );
  connect(chanbox,SIGNAL(valueChanged(int)),SLOT(updateChan(int)));

  vol = new QSpinBox(-99,99,1,this);
  vol->setFrameStyle( QFrame::Panel | QFrame::Raised );
  vol->setGeometry(468,0,54,hh+1);
  vol->setValue(track->gVolume());
  vol->setBackgroundMode( PaletteMid );
  connect(vol,SIGNAL(valueChanged(int)),SLOT(updateVol(int)));

  del = new QSpinBox(-99,99,1,this);
  del->setFrameStyle( QFrame::Panel | QFrame::Raised );
  del->setGeometry(522,0,54,hh+1);
  del->setValue(track->gDelay());
  del->setBackgroundMode( PaletteMid );
  del->setEnabled(FALSE); // DOIT: delay!
  connect(del,SIGNAL(valueChanged(int)),SLOT(updateDel(int)));

  // show();
}

QtTrack::~QtTrack() {

}

QtMain * QtTrack::gQtMain() {
  return (QtMain*) track->gMain()->gInterface();
}

KbTrack * QtTrack::gTrack() {
  return track;
}

void QtTrack::erase() {
  delete this;
}

void QtTrack::update() {
  mute->update(); track->gMain()->muteTrack(track->trackNum());
  type->update();
  lock->update();
  trname->setText(track->gName());
  instbox->setCurrentItem(track->gProgram());
  output->setCurrentItem(track->gOutput());
  chanbox->setValue(track->gChannel());
  vol->setValue(track->gVolume());
  del->setValue(track->gDelay());
  
  if (track->gToggle(1)==FALSE && track->gToggle(2)==FALSE) { // type & lock are false!
    chanbox->setEnabled(TRUE);
    instbox->setEnabled(TRUE);
    output->setEnabled(TRUE);
  } else {
    chanbox->setEnabled(FALSE);
    instbox->setEnabled(FALSE);
    output->setEnabled(FALSE);
  }
  if (track->gToggle(2)==FALSE) { // lock is false
    vol->setEnabled(TRUE);
    del->setEnabled(TRUE);
  } else {
    vol->setEnabled(FALSE);
    del->setEnabled(FALSE);
  }
}

int QtTrack::gProgram() {
  // return instbox->getCurrentItem();
}

void QtTrack::sProgram(int n) {
  track->sProgram(n);
  instbox->setCurrentItem(n);
}

void QtTrack::updateChan(int n) {
  track->sChannel(n);
}

void QtTrack::updateInst(int n) {
  track->sProgram(n);
}

void QtTrack::updateOut(int n) {
  track->sOutput(n);
}

void QtTrack::updateVol(int n) {
  track->sVolume(n);
}

void QtTrack::updateDel(int n) {
  track->sDelay(n);
}



int QtTrack::height() {
  return ((QtMain*)track->gMain()->gInterface())->gTrackHeight();
}

int QtTrack::width() {
  return ((QtMain*)track->gMain()->gInterface())->gTrackWidth();
}

void QtTrack::adjustHeight() {
  int n = track->trackNum()+1;
  int hh = height();
  setGeometry(0,n*(1+hh),width(),hh);
  mute->setGeometry(0,0,24,hh+1);
  type->setGeometry(24,0,24,hh+1);
  lock->setGeometry(48,0,24,hh+1);
  trname->setGeometry(72,0,96,hh+1);
  instbox->setGeometry(168,0,160,hh+1);
  output->setGeometry(328,0,96,hh+1);
  chanbox->setGeometry(424,0,44,hh+1);
  vol->setGeometry(468,0,54,hh+1);
  del->setGeometry(522,0,54,hh+1);
}



// *******************************************************
//
// Toggle Buttons
// ==============
//


TogBut::TogBut( int n, KbTrack * kbtrack, QPixmap * pic, QPixmap * pic2, QWidget * parent=0, const char * name=0 )
  : QLabel( parent, name ), num(n), track(kbtrack), qttrack((QtTrack*)parent), icon(pic), icon2(pic2)
{
  if (icon2==0) icon2 = new QPixmap();
  setAlignment(AlignCenter);
  setPixmap( *icon2 );
}

void TogBut::mousePressEvent( QMouseEvent * mouse ) {
  if (mouse->button()==LeftButton) {
    if (track->gToggle(num)==TRUE) {
      // Off
      setPixmap( *icon2 );
      track->sToggle(num,FALSE);
    } else {
      // On
      setPixmap( *icon );
      setAlignment(AlignCenter);
      track->sToggle(num,TRUE);
    }
    qttrack->update();
  }
}

void TogBut::update() {
  if (track->gToggle(num)==TRUE) {
    // On
    setPixmap( *icon );
  } else {
    // Off
    setPixmap( *icon2 );
  }
}








#endif
